d35 = {}
d35.Version = '1.00.20'
--#################################################################################################################################################
-- #############   Settings and Globals																								                                              ############### 
--#################################################################################################################################################
--
do
  d35.soundsFolder ='d35Sounds'
  d35.messageBeep = '204521__redoper__roger-beep.ogg'
  d35.CardinalDirections = {
							{deg =    0, name = 'North'},
							{deg =   45, name = 'North-East'},
							{deg =   90, name = 'East'},
							{deg =  135, name = 'South-East'},
							{deg =  180, name = 'South'},
							{deg =  225, name = 'South-West'},
							{deg =  270, name = 'West'},
							{deg =  315, name = 'North-West'}
						}

  d35.formations_FixedWing = {
            {name = 'Line Abreast - Close', optionValue = 65537}
        ,   {name = 'Line Abreast - Open', optionValue = 65538}
        ,   {name = 'Line Abreast - GroupClose', optionValue = 65539}
        
        ,   {name = 'Trail - Close', optionValue = 131073}
        ,   {name = 'Trail - Open', optionValue = 131074}
        ,   {name = 'Trail - GroupClose', optionValue = 131075}
        
        ,   {name = 'Wedge - Close', optionValue = 196609}
        ,   {name = 'Wedge - Open', optionValue = 196610}
        ,   {name = 'Wedge - GroupClose', optionValue = 196611}
        
        ,   {name = 'Echelon Right - Close', optionValue = 262145}
        ,   {name = 'Echelon Right - Open', optionValue = 262146}
        ,   {name = 'Echelon Right - GroupClose', optionValue = 262147}
        
        ,   {name = 'Echelon Left - Close', optionValue = 327681}
        ,   {name = 'Echelon Left - Open', optionValue = 327682}
        ,   {name = 'Echelon Left - GroupClose', optionValue = 327683}
        
        ,   {name = 'Finger Four - Close', optionValue = 393217}
        ,   {name = 'Finger Four - Open', optionValue = 393218}
        ,   {name = 'Finger Four - GroupClose', optionValue = 393219}        
           
        ,   {name = 'Spread - Close', optionValue = 458753}
        ,   {name = 'Spread - Open', optionValue = 458754}
        ,   {name = 'Spread - GroupClose', optionValue = 458755}
            
      }




  d35.globalConfig = {
          	nextID = 0
        }


  d35.soundsDictionary = {}
  --[[
      d35.soundsDictionary[word] = {duration = xx, soundFile = xx} ** _soundFile must be in d35.soundsFolder
  --]]

end
--
--#################################################################################################################################################
-- #############   Functions 																										                                                    ############### 
--#################################################################################################################################################
--
function d35.addSound(_word,_soundFile,_duration)
  local row = {duration = _duration, soundFile = _soundFile}
  if not d35.soundsDictionary[string.lower(_word)] then
    d35.soundsDictionary[_word] = row  
  else
    env.info(string.format(' >>>d35.addSound($s, %s, %s) already in the dictionary. Cannot be added', _word,_soundFile,_duration),false)
  end
end
--===================================================================================================================================================
function d35.playSoundToGroup(_GroupID,_soundTable)
  --[[
      _soundTable[i] = {  --** i = sequence 
                            cmd = 'play' | number of seconds to pause
                          , word 
                        }
  ]]--
  --env.info(string.format(' >>>d35.playSoundToGroup() preparing table for %s', _GroupID),false)
  local prepTable, totalDuration = d35.prepareSoundTable(_soundTable)
  if d35.rowCount(prepTable) > 0  then
    --env.info(string.format(' >>>d35.playSoundToGroup() playing table for %s', _GroupID),false)
    d35.playSoundTable(_GroupID,prepTable)
  end
  return totalDuration
end-- function
--===================================================================================================================================================
function d35.prepareSoundTable(_soundTable)
  local ret = {}
  local t = 0
  if d35.rowCount(_soundTable) > 0 then
      i = 1
      for _,r in pairs(_soundTable) do
        if string.lower(r.cmd) == 'play' then
          if  d35.soundsDictionary[string.lower(r.word)] then
            local row = {
                      sound = d35.soundsFolder..'/'..d35.soundsDictionary[string.lower(r.word)].soundFile
                    , time = t
                  }
            table.insert(ret,row)
            t = t + d35.soundsDictionary[string.lower(r.word)].duration
            i = i + 1
            --env.info(string.format(' >>>d35.prepareSoundTable() adding "%s" for %s (%s)', r.word, d35.soundsDictionary[string.lower(r.word)].duration, row.sound),false)
          else
            env.info(string.format(' >>>d35.prepareSoundTable() "%s" is not in the dictionary. Cannot be played', r.word),false)
          end 
        else -- it is a pause
          --env.info(string.format(' >>>d35.prepareSoundTable() adding pause for %s sec.', r.cmd),false)
          t = t + r.pause 
          i = i + 1
        end
      end
  end
  env.info(string.format(' >>>d35.prepareSoundTable() return %s <> %s', d35.rowCount(ret),t),false)
  return ret, t
end-- function
--===================================================================================================================================================
function d35.playSoundTable(_recipient,_soundTable)
  --env.info(string.format(' >>>d35.playSoundTable()  for %s', _recipient),false)
  for _,row in pairs(_soundTable) do
   -- env.info(string.format(' >>> d35.playSoundTable() scheduling "%s" for %s in %s sec.',row.sound, _recipient, row.time),false)
    timer.scheduleFunction( d35.playSound, {_recipient, row.sound},  timer.getTime() + row.time + 0.1 )
  end
end-- function
--===================================================================================================================================================
function d35.playSound(_params,time)
  local _recipient = _params[1]
  local _sound = _params[2]
  env.info(string.format(' >>> d35.playSound() playing "%s" for %s',_sound, _recipient),false)
  if _recipient == 'all' then
     trigger.action.outSound(_sound)
  else
    trigger.action.outSoundForGroup(_recipient,_sound)
  end
  return nil
end -- function
--===================================================================================================================================================
function d35.getObjectByName(_objectName)
	local objectType = nil
	local o = Unit.getByName(_objectName)
	
	if not o then
		-- check if it is a static
		o =  StaticObject.getByName(_objectName)
		if o then
			objectType = 'static'
		end 
	else
		objectType = 'unit'
	end
	
return o, objectType
end -- function
--===================================================================================================================================================
function d35.isAlive(_objectName)
	local ret = false
	-- check if the object is an unit
	local o = Unit.getByName(_objectName)
	if not o then
		-- check if it is a static
		o =  StaticObject.getByName(_objectName)
	end
	
	if o then
		if o:getLife() > 1 then
			ret = true
		end
	end
	return ret 
end -- function
--===================================================================================================================================================
function d35.isGroupAlive(_groupName) 
	local r = false
	local g = Group.getByName(_groupName)
	if g then
		for _,unit in pairs(g:getUnits()) do
			if unit:getLife() >= 1 then
				r = true
				break
			end
		end --for _,unit in pairs(g) do
	end --if g then
	return r
end -- function
--===================================================================================================================================================
function d35.isUnitGroupAlive(_unitsTable) 
	local r = false
	if d35.rowCount(_unitsTable) > 0 then
		for _,unit in pairs(_unitsTable) do
      local u = Unit.getByName(unit)
      if u then
        if u:getLife() >= 1 then
          r = true
          break
        end
      end
		end --for _,unit in pairs(g) do
	end --if d35.rowCount(_unitsTable) > 0 then
	return r
end -- function
--===================================================================================================================================================
function d35.groupHasTarget(_groupName)
  local grp = Group.getByName(_groupName)
  if grp then
    local units = grp:getUnits()
    if d35.rowCount(units) > 0 then
      for _,u in pairs(units) do
        local status,uController = pcall(function(o) return o:getController() end, u)
        if status then
          if d35.rowCount(uController:getDetectedTargets()) > 0 then
            return true
          end -- if d35.rowCount(uController:getDetectedTargets()) > 0 then
        end--if status then
      end--for _,u in pairs(unit) do
    end--if d35.rowCount(units) > 0 then
  end--if grp then
  return false
end -- function
--===================================================================================================================================================
function d35.messageToGroup(_groupName,_messageText,_displayTime)
	if _displayTime == nil then
		_displayTime = 5
	end 
	local _group = Group.getByName(_groupName)
	local _groupId = nil 
	local _callSign = nil 

	if _group then
		 _groupId = _group:getID()
		 local _units = _group:getUnits()
		 _callSign = _units[1]:getCallsign()

	end
	if d35.messageBeep ~= '' then
		if _groupId then 
			trigger.action.outSoundForGroup(_groupId, d35.messageBeep)
		--else
			--trigger.action.outSound(d35.messageBeep)
		end
	end
	local _msg = _messageText
	if _groupId then 
		trigger.action.outTextForGroup(_groupId, _msg, _displayTime)
	--else
		--trigger.action.outText( _msg, _displayTime)
	end
	return true
end
--===================================================================================================================================================
function d35.getDirection(_point1, _point2)
    local tgtBearing
    local xUnit = _point1.x
    local yUnit = _point1.z
    local xZone = _point2.x
    local yZone = _point2.z
    
    local xDiff = xUnit - xZone
    local yDiff = yUnit - yZone
    
    local tgtAngle = math.deg(math.atan(yDiff/xDiff))
    
    if xDiff > 0 then 
    tgtBearing = 180 + tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle > 0 then
    tgtBearing = tgtAngle 
    end
    
    if xDiff < 0 and tgtAngle < 0 then
    tgtBearing = 360 + tgtAngle
    end
    
   
    return tgtBearing
end
--===================================================================================================================================================
function d35.angleSide(_refAngle,_angle)
  -- returns 1 if _angle is to the RIGHT of _refAngle
  -- returns -1 if _angle is to the LEFT of _refAngle
  -- returns 0 if _angle is equal to _refAngle
  
  local difference = _angle - _refAngle

    if (difference < -180.0) then
      difference = difference + 360.0
    elseif ( difference > 180.0) then
      difference =  difference - 360.0
    end
    
    if (difference > 0.0) then
      return 1
    end
    if (difference < 0.0) then
      return -1
    end
    
    return 0
  
end
--===================================================================================================================================================
function d35.angleDiff(_A,_B)
  local a = _B - _A
  a = (a + 180) % 360 - 180
  return a
end -- function
--===================================================================================================================================================
function d35.checkDirection(_dir,_dir1,_dir2)
  local x = mist.utils.toRadian(_dir)
  local a = mist.utils.toRadian(_dir1)
  local b = mist.utils.toRadian(_dir2)
  local ret = d35.distanceBetweenAngles(a, b) >= d35.distanceBetweenAngles(a, x)
  env.info(string.format('>>> d35.checkDirection: %s in [%s,%s]: %s',_dir,_dir1,_dir2,tostring(ret)),false)
  return  ret
end -- function
--===================================================================================================================================================
function d35.distanceBetweenAngles(a, b) 
  -- in radians
    local distance = b - a
    if (a > b) then
       distance = distance + (2*3.14151692)
    end
    return distance
end -- function
--===================================================================================================================================================
function d35.GetCardinalDirection(_direction)
    local dir = d35.fixDirection(_direction)
 		if dir >= 0 and dir <  22 then
				return  'North'
			elseif dir >= 22 and dir <  68 then
				return  'North East'
			elseif dir >= 68 and dir <  113 then
				return  'East'
			elseif dir >= 113 and dir <  158 then
				return  'South East'
			elseif dir >= 158 and dir <  203 then
				return  'South'
			elseif dir >= 203 and dir <  248 then
				return  'South West'
			elseif dir >= 248 and dir <  293 then
				return 'West'
			elseif dir >= 293 and dir <  338 then
				return  'North West'
			elseif dir >= 338 and dir <=  360 then
				return  'North'
		end 
end
--===================================================================================================================================================
function d35.GetCardinalDirectionShort(_direction)
    local dir = d35.fixDirection(_direction)
 		if dir >= 0 and dir <  22 then
				return  'N.'
			elseif dir >= 22 and dir <  68 then
				return  'NE.'
			elseif dir >= 68 and dir <  113 then
				return  'E.'
			elseif dir >= 113 and dir <  158 then
				return  'SE.'
			elseif dir >= 158 and dir <  203 then
				return  'S.'
			elseif dir >= 203 and dir <  248 then
				return  'SW.'
			elseif dir >= 248 and dir <  293 then
				return 'W.'
			elseif dir >= 293 and dir <  338 then
				return  'NW.'
			elseif dir >= 338 and dir <=  360 then
				return  'N.'
		end 
end
--===================================================================================================================================================
function d35.rowCount(_tableName)
	local r = 0

		if next(_tableName) == nil then
			return r
		elseif _tableName == {} then
			return r
		else
			if _tableName then
				for _,_ in pairs(_tableName) do
					r = r + 1
				end
			end
		end

	return r
end -- function
--===================================================================================================================================================
function d35.inTable(_value,_tableName)
	local r = true
	for _,j in pairs(_tableName) do
		if j == _value then
			return true
		end
	end
	return false
	
end -- function
--===================================================================================================================================================
function d35.tableConcat(_table1,_table2)
   for i=1,#_table2 do
      _table1[#_table1+1] = _table2[i]
   end
   return _table1
end --function
--===================================================================================================================================================
function d35.getCoordinatesStrings(_position, _mgrsAccuracy, _dmAccuracy, _dmsAccuracy)
	local lat, lon, alt_m = coord.LOtoLL(_position)
	local mgrs = mist.tostringMGRS(coord.LLtoMGRS(coord.LOtoLL(_position)), _mgrsAccuracy):gsub(string.char(9),' ')
	local dm = mist.tostringLL(lat, lon, _dmAccuracy):gsub(string.char(9),''):gsub(' ','°'):gsub('N°','N '):gsub('S°','S ')
	local dms = mist.tostringLL(lat, lon, _dmsAccuracy,true):gsub(string.char(9),''):gsub(' ','°'):gsub('N°','N '):gsub('S°','S ')
	local alt_ft = mist.utils.metersToFeet(alt_m)

	return mgrs, dm, dms, alt_m, alt_ft
end -- function
--===================================================================================================================================================
function d35.getUnitNames(_groupName)
	local list = {}
	local g = Group.getByName(_groupName)
	if g then
		for _,unit in pairs(g:getUnits()) do
			table.insert(list,unit:getName() )
		end --for _,unit in pairs(g) do	
	end
	return list
end --function
--===================================================================================================================================================
function d35.setGroupOption(_groupName, _option,_value) 
	local con = Group.getByName(_groupName):getController()
	con:setOption(_option, _value) --hold fire
end -- funtion
--===================================================================================================================================================
function d35.getGroupHealth(_groupName)
	local h = 0
	local n = 0
	local g = Group.getByName(_groupName)
	
	if g then
		local ut = g:getUnits()
		for index, data in pairs(ut) do
			local a =  Unit.getLife(data) 	
			h = h + a
			if a >= 1 then
				n = n + 1
			end
		end
	end
	return h,n
end-- function
--===================================================================================================================================================
function d35.getUnitGroupHealth(_unitsTable) 
	local r = 0
	if d35.rowCount(_unitsTable) > 0 then
		for _,unit in pairs(_unitsTable) do
      local u = Unit.getByName(unit)
      if u then
        if u:getLife() >= 1 then
          r = r + u:getLife() 
        end
      end
		end --for _,unit in pairs(g) do
	end --if d35.rowCount(_unitsTable) > 0 then
	return r
end -- function
--===================================================================================================================================================
function d35.getGroupPOS(_groupName)
	local r = nil
	local g = Group.getByName(_groupName)
	if g then
		local t = d35.getUnitNames(_groupName)
		 r = mist.getAvgPos(t)
	end --if g then
	return r
end -- function
--===================================================================================================================================================
function d35.getNextID()
    local id = 0
	
	-- Try MOOSE
	--local _status,id =  pcall(function()
	--						return {UTILS.GetMarkID()}
	--					end)	
	--if not _status then
		-- Try MIST
	--	local _status,id =  pcall(function()
	--							return {mist.marker.getNextId()}
	--						end)	
	--	if not _status then	
			-- Use MGT
			d35.globalConfig.nextID = d35.globalConfig.nextID+1
			id = d35.globalConfig.nextID 
	--	end -- MIST
	
--	end -- MOOSE

	return  id -- MGT.globalConfig.nextID
end -- function
--===================================================================================================================================================
function d35.distancePoint2Line(_point,_linePoint1,_linePoint2) 
  local dist = 0
  local px = _point.x
  local py = _point.y
  local x1 = _linePoint1.x
  local y1 = _linePoint1.y
  local x2 = _linePoint2.x
  local y2 = _linePoint2.y
 
  local dx,dy = x2-x1,y2-y1
  local length = math.sqrt(dx*dx+dy*dy)
  dx,dy = dx/length,dy/length
  local posOnLine = dx*(px-x1) + dy*(py-y1)
  if posOnLine < 0 then
    -- first end point is closest
    dx,dy = px-x1,py-y1
    dist = math.sqrt(dx*dx+dy*dy)
  elseif posOnLine > length then
    -- second end point is closest
    dx,dy = px-x2,py-y2
    dist = math.sqrt(dx*dx+dy*dy)   
  else
    -- point is closest to some part in the middle of the line
    dist = math.abs( dy*(px-x1) - dx*(py-y1))
  end
  return dist
end --function
--===================================================================================================================================================
function d35.fixDirection(_angle)
  local fixed_angle = _angle
  if _angle < 0 then
    fixed_angle = 360 + _angle
  elseif _angle > 359 then
    fixed_angle =  _angle - 360
  end
  return fixed_angle
end -- function
--===================================================================================================================================================
function d35.chance(_prob)
  local p = mist.random(1,100)
  local r = false
  if (p <= _prob) then
    r = true
  end
   env.info(string.format("d35.chance() p: %s <= param: %s = %s",p,_prob,tostring(r)),false)
  return r
end
--===================================================================================================================================================
function d35.trim(s)
   return (s:gsub("^%s*(.-)%s*$", "%1"))
end
--===================================================================================================================================================
function d35.ConvertNumToWords(n)
  
	-- Convert a number to words
	-- Author: Nick Gammon
	-- Date: 18th March 2010

	-- Example of use:
	-- words  = ConvertNumToWords('94921277802687490518')

	-- Return nil and an error message so you can check for failure,
	-- or assert, eg. words = assert(ConvertNumToWords('2687490518'))
	
	-- See: http://www.gammon.com.au/forum/?id=10155
	
	inverse_units = {
		"vigintillion ",     -- 10^63
		"novemdecillion ",   -- 10^60
		"octodecillion ",    -- 10^57
		"septendecillion ",  -- 10^54
		"sexdecillion ",     -- 10^51
		"quindecillion ",    -- 10^48
		"quattuordecillion ",-- 10^45
		"tredecillion ",     -- 10^42
		"duodecillion ",    -- 10^39
		"undecillion ",     -- 10^36
		"decillion ",       -- 10^33
		"nonillion ",       -- 10^30
		"octillion ",       -- 10^27
		"septillion ",      -- 10^24
		"sextillion ",      -- 10^21
		"quintillion ",     -- 10^18
		"quadrillion ",     -- 10^15
		"trillion ",        -- 10^12
		"billion ",         -- 10^9
		"million ",         -- 10^6
		"thousand ",        -- 10^3
	} -- inverse_units
  
	inverse_numbers = {
		"one ",
		"two ",
		"three ",
		"four ",
		"five ",
		"six ",
		"seven ",
		"eight ",
		"nine ",
		"ten ",
		"eleven ",
		"twelve ",
		"thirteen ",
		"fourteen ",
		"fifteen ",
		"sixteen ",
		"seventeen ",
		"eighteen ",
		"nineteen ",
		"twenty ",
		[30] = "thirty ",
		[40] = "forty ",
		[50] = "fifty ",
		[60] = "sixty ",
		[70] = "seventy ",
		[80] = "eighty ",
		[90] = "ninety ",
	}  -- inverse_numbers
 
	

	s = tostring(n)
  
	-- preliminary sanity checks
	c = string.match (s, "%D")
	if c then
		return nil, "Non-numeric digit '" .. c .. "' in number"
	end -- if

	if #s == 0 then
		return nil, "No number supplied"
	elseif #s > 66 then
		return nil, "Number too big to convert to words"
	end -- if
  
	-- make multiple of 3
	while #s % 3 > 0 do
		s = "0" .. s
	end -- while
      
	result = ""
	start = #inverse_units - (#s / 3) + 2
  
	for i = start, #inverse_units do
		group = tonumber(string.sub (s, 1, 3))
	if group > 0 then
		result = result .. d35.ConvertTo999 (group) .. inverse_units [i]
	end -- if not zero
		s = string.sub(s, 4)    
	end -- for
  
	result = result .. d35.ConvertTo999(tonumber (s)) 

	if result == "" then
		result = "zero"
	end -- if
  
	return (string.gsub(result, " +$", ""))  -- trim trailing spaces

end -- ConvertNumToWords
--===================================================================================================================================================
function d35.ConvertTo999(n)
  
	-- Convert a number to words
	-- Author: Nick Gammon
	-- Date: 18th March 2010

	-- Example of use:
	-- words  = ConvertNumToWords('94921277802687490518')

	-- Return nil and an error message so you can check for failure,
	-- or assert, eg. words = assert(ConvertNumToWords('2687490518'))
	
	-- See: http://www.gammon.com.au/forum/?id=10155
	
	if n <= 0 then
		return ""
	end -- if zero
  
	hundreds = math.floor(n / 100)
	tens = math.floor(n % 100)
	result = ""

	-- if over 99 we need to say x hundred  
	if hundreds > 0 then
  
    result = inverse_numbers[hundreds] .. "hundred "
    if tens == 0 then
      return result
    end -- if only a digit in the hundreds column
  
	-- to have "and" between things like "hundred and ten"
	-- uncomment the next line
	--result = result .. "and "

	end -- if
  
	-- up to twenty it is then just five hundred (and) fifteen
	if tens <= 20 then
		return result .. inverse_numbers[tens] 
	end -- if

	-- otherwise we need: thirty (something)
	result = result .. inverse_numbers[math.floor(tens / 10) * 10] 
  
	-- get final digit (eg. thirty four)
	local digits = math.floor(n % 10)

	-- to put a hyphen between things like "forty-two" 
	-- uncomment the WITH HYPHEN line and 
	-- comment out the NO HYPHEN line

	if digits > 0 then
		 result = result ..  inverse_numbers [digits]  -- NO HYPHEN
		--result = string.sub(result, 1, -2) .. "-" ..  inverse_numbers[digits]  -- WITH HYPHEN
	end -- if 

	return result
  
end -- ConvertTo999
--===================================================================================================================================================
function d35.stringParse(inputstr, sep)
	if sep == nil then
			sep = "%s"
	end
	local t={}
	for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
			table.insert(t, str)
	end
	return t
end
--===================================================================================================================================================



--
--#################################################################################################################################################
-- #############   main body 																										                                                    ############### 
--#################################################################################################################################################
do -- main body
	env.info(' >>> d35lib.lua - v'..d35.Version..' loaded!<<<',false)
end